var db = require("../db/pool");
var _ = require('lodash');

/**
 * 保存Json到指的的表中。
 * @param {*} json 要保存的数据
 * @param {*} table  数据表名称
 * @param {*} key 主键名称
 */
var saveJsonToDb = (json, table, key) => {
    return new Promise((resovle, reject) => {
        var sql = "";
        if (_.has(json, key) && _.get(json, key, 0) > 0) {
            var jsonClone = Object.assign({},json);
            _.unset(jsonClone, key);
            var where = {};
            where[key] = _.get(json, key);;
            // update
            sql = db.sql.$update({
                $table: table,
                $set: jsonClone,
                $where: where
            });
        } else {
            _.unset(json, key);
            // insert
            sql = db.sql.$insert({
                $table: table,
                $documents: json
            });
        }
        db.query(sql.sql, sql.values)
            .then((rows) => {
                // console.log(JSON.stringify(rows));
                if (rows.insertId > 0) {
                    json[key] = rows.insertId;
                }
                resovle(json);
            }).catch((err) => {
                reject(JSON.stringify(err));
            });
    });
};

/**
 * 通过json条件获取数据库内数据列表
 * @param {*} json 查询条件
 * @param {*} table 数据表名称
 * @param {*} items 要查询的字段
 * @param {Object} extra 额外值
 */
var listDbByJson = (json, table, items, extra = {}) => {
    return new Promise((resovle, reject) => {
        var query = {};
        //整理字段
        if (_.isString(items) && items != "*") {
            //单独项目查询
            var map = {};
            map[items] = true;
            query['$columns'] = map;
        } else if (_.isArray(items)) {
            var map = {};
            items.forEach((v) => {
                map[v] = true;
            })
            query['$columns'] = map;
        } else if (_.isObject(items)) {
            var citems = Object.assign({}, items);
            query = _.merge(query, citems);
        }
        //添加额外查询条件 比如order by等
        var extraClone = Object.assign({}, extra);
        query = _.merge(query, extraClone);

        var cloned = Object.assign({}, json);
        query['$where'] = cloned;
        query['$from'] = table;

        var sql = db.sql.$select(query);
        db.query(sql.sql, sql.values)
            .then((rows) => {
                resovle(rows);
            }).catch((err) => {
                reject(JSON.stringify(err));
            });
    });
}

/**
 * 根据条件获取数据长度
 * @param {*} json 
 * @param {*} table 
 * @param {*} key 
 */
var countByJson = (json, table, key) => {
    return new Promise((resovle, reject) => {
        listDbByJson(json, table, { len: { $count: key } })
            .then((res) => {
                if (_.isEmpty(res)) {
                    resovle(0)
                }
                resovle(res[0]['len']);
            }).catch((err) => {
                reject(JSON.stringify(err));
            });
    });
}


/**
 * 获取分页数据
 * @param {*} json  查询条件
 * @param {*} table  
 * @param {*} items 
 * @param {*} key 
 * @param {*} page 
 * @param {*} pageSize 
 */
var pagedDbByJson = (json, table, items, key, page = 1, pageSize = 10, orderby = "") => {
    return new Promise((resovle, reject) => {
        countByJson(json, table, key)
            .then((len) => {
                if (len > 0) {
                    let extra = {
                        $limit: pageSize,
                        $offset: (page - 1) * pageSize,
                    }
                    if (!_.isEmpty) {
                        extra['$orderby'] = orderby;
                    }
                    return listDbByJson(json, table, items, extra).then((data) => {
                        resovle({
                            data,
                            paged: {
                                page,
                                pageSize,
                                total: len
                            }
                        });
                    }).catch((err) => {
                        throw new Error(err);
                    })
                } else {
                    resovle({
                        data: [],
                        paged: {
                            page,
                            pageSize,
                            total: 0
                        }
                    });
                }
            }).catch((err) => {
                reject(JSON.stringify(err));
            })
    });
}

module.exports = {
    saveJsonToDb,
    listDbByJson,
    countByJson,
    pagedDbByJson,
}